'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = findMatchingVNodes;

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function getSelectors(selector) {
  var selectors = selector.split(' ');
  return selectors.reduce(function (list, sel) {
    return list.concat(sel);
  }, []);
}

function findAllVNodes(vNode) {
  var nodes = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
  var ignoreFirstNode = arguments[2];
  var depth = arguments[3];

  if (!ignoreFirstNode) {
    nodes.push(vNode);
  }

  if (depth === 0) {
    return;
  }

  if (vNode.children && vNode.children.length > 0) {
    for (var i = 0; i < vNode.children.length; i++) {
      findAllVNodes(vNode.children[i], nodes, false, depth ? depth - 1 : undefined);
    }
  }

  if (vNode.child) {
    findAllVNodes(vNode.child._vnode, nodes, false, depth);
  }

  return nodes;
}

function getMatchingVNodes(vNode, selector, ignoreFirstNode, depth) {
  var nodes = findAllVNodes(vNode, [], ignoreFirstNode, depth);
  return nodes.filter(function (node) {
    if (node.elm && node.elm.matches) {
      return node.elm.matches(selector);
    }
    return false;
  });
}

function recurseGetMatchingVNodes(vNodes, selectors, ignoreFirstNode) {
  var nodes = [];
  var newSelectors = void 0;

  if (selectors[0] === '>') {
    vNodes.forEach(function (node) {
      return nodes.push.apply(nodes, _toConsumableArray(getMatchingVNodes(node, selectors[1], true, 1)));
    });
    newSelectors = selectors.slice(2);
  } else {
    vNodes.forEach(function (node) {
      return nodes.push.apply(nodes, _toConsumableArray(getMatchingVNodes(node, selectors[0], ignoreFirstNode)));
    });
    newSelectors = selectors.slice(1);
  }

  if (newSelectors.length < 1) {
    return nodes;
  }

  return recurseGetMatchingVNodes(nodes, newSelectors, true);
}

function removeDuplicateNodes(vNodes) {
  var uniqueNodes = [];
  vNodes.forEach(function (vNode) {
    var exists = uniqueNodes.some(function (node) {
      return vNode.elm === node.elm;
    });
    if (!exists) {
      uniqueNodes.push(vNode);
    }
  });
  return uniqueNodes;
}

function findMatchingVNodes(vNode, selector) {
  var selectorsArray = getSelectors(selector);
  var nodes = void 0;

  if (selectorsArray.length > 1) {
    nodes = recurseGetMatchingVNodes([vNode], selectorsArray);
  } else {
    nodes = getMatchingVNodes(vNode, selector);
  }

  return removeDuplicateNodes(nodes);
}